<?php
    class enhance_Module extends ServerModule
    {
        public $api;

        function __construct($server,$options=[])
        {
            $this->_name = __CLASS__;
            $this->force_setup  = false;
            parent::__construct($server,$options);
        }

        protected function define_server_info($server=[])
        {
            if(!file_exists(__DIR__.DS.'api'.DS.'vendor'.DS.'autoload.php'))
            {
                $source_url         = "https://wisecp.com/files/modules/EnhanceApi.zip";
                $source_archive     = ROOT_DIR."temp".DS."EnhanceApi.zip";
                $download_zip       = Updates::download_remote_file($source_url,$source_archive);

                if(!$download_zip || !file_exists($source_archive))
                    die(Utility::jencode([
                        'status' => "error",
                        'message' => "Resource files cannot be downloaded from the server.",
                    ]));

                if(!class_exists("ZipArchive"))
                    die(Utility::jencode([
                        'status' => "error",
                        'message' => "ZipArchive class not found",
                    ]));

                $zip = new ZipArchive;

                if ($zip->open($source_archive) === TRUE) {
                    $zip->extractTo(__DIR__);
                    $zip->close();
                    FileManager::file_delete($source_archive);
                }
                else
                    die(Utility::jencode([
                        'status' => "error",
                        'message' => "ZipArchive can not open file",
                    ]));

            }

            require __DIR__.DS.'api'.DS.'vendor'.DS.'autoload.php';
            $this->api = $this->get_api_client();
        }

        public function config_options($data=[])
        {
            return [
                'package_id'          => [
                    'wrap_width'        => 100,
                    'name'              => "Enhance package ID",
                    'description'       => "Obtain this from the URL when visiting the packages page in your Enhance admin panel",
                    'type'              => "text",
                    'value'             => $data["package_id"] ?? "",
                    'placeholder'       => "",
                ]
            ];
        }

        private function fetch_or_create_org($params=[])
        {
            $api = $this->api;
            $field = User::getInfo($this->user["id"],['enhanceOrgId']);

            $orgId = $field["enhanceOrgId"];

            if ($orgId != "") {
                return $orgId;
            } else {
                $orgId = $this->createCustomer($params);
                if (strlen($orgId) < 10) throw new Exception("Org ID appears too short");

                User::setInfo($this->user["id"],['enhanceOrgId' => $orgId]);

                return $orgId;
            }
        }

        private function fetch_org($params = [])
        {
            $field = User::getInfo($this->user["id"],['enhanceOrgId']);

            $orgId = $field["enhanceOrgId"];

            if ($orgId == "") {
                throw new Exception("Unable to find client organisation");
            }

            return $orgId;
        }

        private function createCustomer($params=[])
        {
            $api = $this->api;

            $new_customer = new \OpenAPI\Client\Model\NewCustomer;

            if ($this->user['company_name'] && $this->user['company_name'] != "")
                $name = $this->user['company_name'];
            else
                $name = $this->user['full_name'];

            $users      = $api['customersClient']->getOrgCustomers($this->server['username']);
            $userList   = $users->getItems() ?: [];

            if($userList) foreach($userList AS $u) if($u->getOwnerEmail() == $this->user["email"]) return $u->getId();

            $new_customer->setName($name);
            $org = $api['customersClient']->createCustomer($this->server['username'], $new_customer);

            $new_login = new \OpenAPI\Client\Model\LoginInfo;

            $new_login->setEmail($this->user['email']);
            $new_login->setName($name);
            $new_login->setPassword($params['password'] ?? Utility::generate_hash(32));

            $login = $api['loginsClient']->createLogin($org['id'], $new_login);

            $new_member = new \OpenAPI\Client\Model\NewMember;
            $new_member->setLoginId($login['id']);
            $new_member->setRoles([\OpenAPI\Client\Model\Role::OWNER]);

            $member = $api['membersClient']->createMember($org['id'], $new_member);
            return $org['id'];
        }

        private function get_api_client()
        {
            $config = OpenAPI\Client\Configuration::getDefaultConfiguration()
                ->setAccessToken($this->server['access_hash'])
                ->setHost("http".($this->server["secure"] ? "s" : "")."://" . $this->server['name'] . "/api")
            ;

            $customersClient = new OpenAPI\Client\Api\CustomersApi(new GuzzleHttp\Client(), $config);
            $subscriptionsClient = new OpenAPI\Client\Api\SubscriptionsApi(new GuzzleHttp\Client(), $config);
            $loginsClient = new OpenAPI\Client\Api\LoginsApi(new GuzzleHttp\Client(), $config);
            $membersClient = new OpenAPI\Client\Api\MembersApi(new GuzzleHttp\Client(), $config);
            $orgsClient = new OpenAPI\Client\Api\OrgsApi(new GuzzleHttp\Client(), $config);
            $websitesClient = new OpenAPI\Client\Api\WebsitesApi(new GuzzleHttp\Client(), $config);

            return array(
                "customersClient" => $customersClient,
                "subscriptionsClient" => $subscriptionsClient,
                "loginsClient" => $loginsClient,
                "membersClient" => $membersClient,
                "orgsClient" => $orgsClient,
                "websitesClient" => $websitesClient,
            );
        }

        public function create($domain = '',array $order_options=[])
        {
            $password       = Utility::generate_hash(24);
            $creation_info  = isset($order_options["creation_info"]) ? $order_options["creation_info"] : [];
            $package_id     = $creation_info["package_id"];

            $order_options["password"] = $password;

            try
            {
                $orgId = $this->fetch_or_create_org($order_options);

                $new_subscription = new \OpenAPI\Client\Model\NewSubscription;
                $new_subscription->setPlanId(intval($package_id));
                $subscription = $this->api['subscriptionsClient']->createCustomerSubscription($this->server['username'], $orgId, $new_subscription);

                $new_website = new \OpenAPI\Client\Model\NewWebsite;
                $new_website->setSubscriptionId($subscription['id']);
                $new_website->setDomain($domain);
                $website = $this->api['websitesClient']->createWebsite($orgId, $new_website);

                /*
                $update_website = new \OpenAPI\Client\Model\UpdateWebsite;
                $update_website->setPhpVersion(\OpenAPI\Client\Model\PhpVersion::PHP81);

                $this->api['websitesClient']->updateWebsite($orgId, $website['id'], $update_website);
                */

                return [
                    'username' => $this->user["email"],
                    'password' => $password,
                    'properties' => [
                        'enhanceSubscriptionId' => $subscription['id']
                    ],
                ];

            }
            catch (Exception $e){
                $this->error = $e->getMessage();
                self::save_log(
                    'Servers',
                    $this->_name,
                    __FUNCTION__,
                    ['order' => $this->order],
                    $e->getMessage(),
                    $e->getTraceAsString()
                );
                return false;
            }
        }

        public function suspend()
        {
            try
            {
                $existing_sub = $this->order["options"]["properties"]["enhanceSubscriptionId"] ?? false;

                if (!is_numeric($existing_sub)) throw new Exception("Subscription not found");

                $orgId = $this->fetch_or_create_org();

                $update_subscription = new \OpenAPI\Client\Model\UpdateSubscription;
                $update_subscription->setIsSuspended(true);

                $this->api['subscriptionsClient']->updateSubscription($orgId, $existing_sub, $update_subscription);

                return true;
            }
            catch (Exception $e){
                $this->error = $e->getMessage();
                self::save_log(
                    'Servers',
                    $this->_name,
                    __FUNCTION__,
                    ['order' => $this->order],
                    $e->getMessage(),
                    $e->getTraceAsString()
                );
                return false;
            }
        }

        public function unsuspend()
        {
            try
            {

                $existing_sub = $this->order["options"]["properties"]["enhanceSubscriptionId"] ?? false;

                if (!is_numeric($existing_sub)) throw new Exception("Subscription not found");

                $orgId = $this->fetch_or_create_org();

                $update_subscription = new \OpenAPI\Client\Model\UpdateSubscription;
                $update_subscription->setIsSuspended(false);

                $this->api['subscriptionsClient']->updateSubscription($orgId, $existing_sub, $update_subscription);

                return true;
            }
            catch (Exception $e){
                $this->error = $e->getMessage();
                self::save_log(
                    'Servers',
                    $this->_name,
                    __FUNCTION__,
                    ['order' => $this->order],
                    $e->getMessage(),
                    $e->getTraceAsString()
                );
                return false;
            }
        }

        public function terminate()
        {
            try
            {

                $existing_sub = $this->order['options']['properties']['enhanceSubscriptionId'] ?? false;
                if (!is_numeric($existing_sub)) throw new Exception("Subscription not found");

                $orgId = $this->fetch_or_create_org();

                // this is a soft deletion
                $this->api['subscriptionsClient']->deleteSubscription($orgId, $existing_sub, "false");


            }
            catch (Exception $e){
                $this->error = $e->getMessage();
                if(stristr($this->error,'not_found')) return true;

                self::save_log(
                    'Servers',
                    $this->_name,
                    __FUNCTION__,
                    ['order' => $this->order],
                    $e->getMessage(),
                    $e->getTraceAsString()
                );
                return false;
            }
            return true;
        }

        public function apply_updowngrade($orderopt=[],$product=[]){
            $p_creation_info        = $product["module_data"];

            try
            {
                $orgId = $this->fetch_org();
                $subscriptionId = $orderopt["properties"]["enhanceSubscriptionId"] ?? false;
                if (!is_numeric($subscriptionId)) throw new Exception("Subscription not found");

                $this->api['subscriptionsClient']->updateSubscription($orgId, $subscriptionId, array("planId" => intval($p_creation_info['package_id'] ?? 0)));

            }
            catch (Exception $e)
            {
                $this->error = $e->getMessage();
                self::save_log(
                    'Servers',
                    $this->_name,
                    __FUNCTION__,
                    ['order' => $this->order],
                    $e->getMessage(),
                    $e->getTraceAsString()
                );
                return false;
            }
        }

        public function use_clientArea_SingleSignOn()
        {
            try
            {

                $orgId = $this->fetch_org();

                $members = $this->api['membersClient']->getMembers($orgId)->getItems();

                $owners = array_filter($members, function ($member) {
                    $roles = $member->getRoles();
                    return is_numeric(array_search("Owner", $roles));
                });

                $owner = array_pop($owners);

                if (!$owner) throw new Exception("Unable to locate organisation owner for SSO");

                $ownerId = $owner->getId();

                $url = str_replace('"','',$this->api['membersClient']->getOrgMemberLogin($orgId, $ownerId));


                Utility::redirect($url);
                echo "Redirecting...";
            }
            catch(Exception $e)
            {
                echo  $e->getMessage();
                return false;
            }
        }

        public function use_adminArea_SingleSignOn()
        {
            return $this->use_clientArea_SingleSignOn();
        }

    }